// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "InstanceAssociationsForm.h"
#include "SelectClassDialog.h"
#include "PropertyPagesDialog.h"
#include "CIMOMHandle.h"
#include "InstancePropertyListForm.h"
#include <gtkmm/liststore.h>
#include <gtkmm/messagedialog.h>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <iostream>
#include <assert.h>

using Glib::RefPtr;
using Glib::ustring;
using namespace Gtk;
using namespace OpenWBEM;

InstanceAssociationsList::InstanceAssociationsList()
{
	RefPtr<ListStore> store = ListStore::create(Columns);
	set_model(store);

	append_column("Relation", Columns.relation);
	append_column("Associated Instance Name", Columns.instanceName);

	Menu::MenuList &menulist = context_menu.items();

	menulist.push_back(Menu_Helpers::MenuElem("_New Association...",
		SigC::slot(*this, &InstanceAssociationsList::on_menu_new_association)));
	menulist.push_back(Menu_Helpers::MenuElem("_Delete Association",
		SigC::slot(*this, &InstanceAssociationsList::on_menu_delete_association)));
}

void
InstanceAssociationsList::setBaseInstance(CIMInstanceHandle baseHandle)
{
	RefPtr<ListStore> store = RefPtr<ListStore>::cast_dynamic(get_model());

	std::vector<CIMInstanceHandle> references = baseHandle->references();

	for (unsigned int i = 0; i < references.size(); i++)
	{
		CIMInstanceHandle refInst = references[i];
		CIMObjectPath cop = refInst->getObjectPath();		

		TreeModel::iterator iter = store->append();
		TreeModel::Row row = *iter;
		
		row[Columns.assocClassName] = cop.getClassName().c_str();
		row[Columns.referenceHandle] = refInst;
		row[Columns.relation] = ustring(cop.getClassName().c_str())
			+ ".?";

		CIMPropertyArray props = refInst->getProperties();
		bool foundReference = false;
		bool foundAssociatedInstance = false;
		for (unsigned int j = 0; j < props.size(); j++)
		{
			CIMProperty prop = props[j];
			if (prop.isKey() && prop.isReference())
			{
				foundReference = true;
				CIMObjectPath mcop = prop.getValue().toCIMObjectPath();
				if (mcop != baseHandle->getObjectPath())
				{
					foundAssociatedInstance = true;
					CIMInstanceHandle handle = CIMInstanceProxy::create(mcop);
					row[Columns.propertyName] = prop.getName().c_str();
					row[Columns.relation] = ustring(cop.getClassName().c_str())
						+ "." + prop.getName().c_str();
					if (prop.getQualifier("DisplayName"))
					{
						CIMValue dispNameValue = prop.getQualifier("DisplayName")
							.getValue();
						if (dispNameValue)
						{
							row[Columns.relation] = ustring(dispNameValue.toString().c_str());
						}
					}
					row[Columns.instanceHandle] = handle;

					CIMProperty prop = handle->getProperty("ElementName");
					row[Columns.instanceName] = prop.getValue().toString().c_str();
				}
			}
		}
		if (!foundReference)
		{
			row[Columns.instanceName] = "(Error: no key reference values)";
			g_message("cop = %s",
				cop.toString().c_str());
		}
		else if (!foundAssociatedInstance)
		{
			row[Columns.instanceName] = "(Error: no other instance in association)";
			g_message("cop = %s",
				cop.toString().c_str());
		}
	}
}

bool
InstanceAssociationsList::on_button_press_event(GdkEventButton *event)
{
	assert(event != NULL);
	
	// do default handler
	TreeView::on_button_press_event(event);

	// check which button is pressed
	if (event->button == 3)
	{
		context_menu.popup(event->button, event->time);
		return true;
	}
	return false;
}

void
InstanceAssociationsList::on_menu_new_association()
{
	try
	{
		// let the user pick the association class
		SelectClassDialog dialog;
		dialog.set_title("Create Instance");
		dialog.set_transient_for(*(Window *) get_toplevel());
		int response = dialog.run();
		dialog.hide();
		
		if (response != RESPONSE_OK)
			return;

		// create a space for holding the new instance
		// (instance is not actually created until user OKs this
		// dialog, thereby "committing" the new instance)
		String className = dialog.get_class_name();
		CIMInstanceHandle handle = CIMInstanceProxy::new_instance(
				CIMOMHandle::getDefaultNamespace(),
				className);

		// let user edit new instance's properties
		PropertyPagesDialog *dlg = new PropertyPagesDialog;
		dlg->set_title("Create Instance");
		dlg->set_transient_for(*(Window *) get_toplevel());
		dlg->add_form(new InstancePropertyListForm(handle),
			"Properties");
		dlg->show();
	}
	catch (const Exception &e)
	{
		MessageDialog dialog(*(Window *) get_toplevel(),
			e.getMessage(), MESSAGE_ERROR);
		dialog.run();
	}
}
	
void
InstanceAssociationsList::on_menu_delete_association()
{
	g_message("delete");
}

InstanceAssociationsForm::InstanceAssociationsForm(CIMInstanceHandle objectref)
	: objectref(objectref)
{
	instances_list.setBaseInstance(objectref);
	instances_list.show();

	instances_scrolled.set_shadow_type(SHADOW_IN);
	instances_scrolled.set_border_width(6);
	instances_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	instances_scrolled.add(instances_list);
}

void
InstanceAssociationsForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
	//instances_list.list_properties(cop, inst);
}

Widget *
InstanceAssociationsForm::getWidget()
{
	return &instances_scrolled;
}

void
InstanceAssociationsForm::applyChanges()
{
}
