// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "InstanceList.h"
#include "CIMOMHandle.h"
#include "PropertyPagesDialog.h"
#include "InstancePropertyListForm.h"
#include "ClassHandlerRegistry.h"

#include <openwbem/OW_Enumeration.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_Reference.hpp>
#include <openwbem/OW_HTTPException.hpp>
#include <assert.h>

using namespace Gtk;
using namespace OpenWBEM;

InstanceList::InstanceList()
{
	this->use_default_instance_activated_handler = true;
}

void
InstanceList::add_text_column(String property_name, Glib::ustring header)
{
	TreeModelColumn<Glib::ustring> string_column;
	Columns.add(string_column);
	append_column(header, string_column);
	
	TextColumn *col = new TextColumn(property_name, string_column);
	column_mappings.push_back(Reference<Column>(col));
}

void
InstanceList::add_constant_text_column(Glib::ustring text, Glib::ustring header)
{
	TreeModelColumn<Glib::ustring> string_column;
	Columns.add(string_column);
	append_column(header, string_column);
	
	String prop_name;
	Column *col = new ConstantColumn(prop_name, string_column, text);
	column_mappings.push_back(Reference<Column>(col));
}

void
InstanceList::remove_all_columns()
{
	TreeView::remove_all_columns();
	column_mappings.clear();
}

void
InstanceList::enum_instances(String className)
{
	new_list();
	try
	{
		std::vector<CIMInstanceHandle> result
			= CIMOMHandle::getAllInstances(className);
		populate_list(result);
	}
	catch (CIMException e)
	{
		g_warning("InstanceList::enum_instances: %s", e.getMessage());
	}
	catch (HTTPException e)
	{
		g_warning("InstanceList::enum_instances: %s", e.getMessage());
	}
}

void
InstanceList::associators(const CIMObjectPath &refObject,
		const String &assocClass)
{
	new_list();
	try
	{
		CIMObjectPathEnumeration resultEnum
			= CIMOMHandle::associatorNamesE(refObject, assocClass);
		std::vector<CIMInstanceHandle> result;
		while (resultEnum.hasMoreElements())
		{
			result.push_back(CIMInstanceProxy::create(resultEnum.nextElement()));
		}
		populate_list(result);

		//result = CIMOMHandle::referencesE(refObject);
	}
	catch (const Exception &e)
	{
		g_warning("%s", e.getMessage());
	}
}

void
InstanceList::new_list()
{
	// create a new list store, using the current columns record
	m_store = ListStore::create(Columns);
	set_model(m_store);
}

void
InstanceList::populate_list(const std::vector<CIMInstanceHandle> &result)
{
	for (unsigned int i = 0; i < result.size(); i++)
	{
		CIMInstanceHandle inst = result[i];
		CIMObjectPath cop = inst->getObjectPath();
		Glib::ustring classname = cop.getClassName().c_str();

		TreeModel::iterator iter = m_store->append();
		TreeModel::Row row = *iter;
			
		row[Columns.classname] = classname;
		row[Columns.inst_handle] = inst;
	
		for (unsigned int i = 0; i < column_mappings.size(); i++)
		{
			column_mappings[i]->setRowData(row, inst);
		}
	}
}

void
InstanceList::on_row_activated(const TreeModel::Path &path,
		TreeViewColumn *column)
{
	TreeModel::iterator iter = m_store->get_iter(path);
	TreeModel::Row row = *iter;
	
	CIMInstanceHandle inst_handle = row[Columns.inst_handle];
	on_instance_activated(inst_handle);
}

void
InstanceList::on_instance_activated(CIMInstanceHandle inst)
{
	CIMObjectPath cop(inst->getObjectPath());

	if (use_default_instance_activated_handler)
	{
		ClassHandler *handler = ClassHandlerRegistry::get_handler(
				cop.getClassName());
		assert(handler != NULL);

		handler->do_default_action(inst, (MainWindow *) get_toplevel());
	}

	signal_activated.emit(cop);
}

CIMObjectPath
InstanceList::get_selected_object()
{
	TreePath path;
	TreeViewColumn *column;
	Glib::RefPtr<TreeModel> model;
	
	get_cursor(path, column);
	if (path.gobj() == NULL)
	{
		return CIMObjectPath(CIMNULL);
	}
	
	model = get_model();
	TreeModel::iterator iter = model->get_iter(path);
	TreeModel::Row row = *iter;

	CIMInstanceHandle inst = row[Columns.inst_handle];
	return inst->getObjectPath();
}

InstanceList::Column::~Column()
{
}

void
InstanceList::TextColumn::setRowData(TreeRow &row, CIMInstanceHandle inst)
{
	CIMProperty prop = inst->getProperty(PropertyName);
	if (prop)
	{
		CIMValue value = prop.getValue();
		if (value)
		{
			String string;
			value.get(string);
			row[ModelColumn] = string.c_str();
		}
	}
	else
	{
		g_warning("Property %s not defined", PropertyName.c_str());
	}
}

void
InstanceList::ConstantColumn::setRowData(TreeRow &row, CIMInstanceHandle inst)
{
	row[ModelColumn] = ConstantText;
}
