// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef INSTANCELIST_H
#define INSTANCELIST_H

#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <openwbem/OW_CIMOMHandleIFC.hpp>
#include <openwbem/OW_CIMInstance.hpp>
#include <vector>
#include "CIMInstanceProxy.h"

/**
 * Displays multiple instances' data in a tabular format.
 * Instances can be double-clicked to bring up their properties window.
 */
class InstanceList : public Gtk::TreeView
{
private:
	Glib::RefPtr<Gtk::ListStore> m_store;

public:
	InstanceList();

	/**
	 * Add a column displaying text data. The added column corresponds
	 * to a CIM property of the instances that get displayed.
	 */
	void add_text_column(OpenWBEM::String property_name, Glib::ustring header);

	/**
	 * Add a column that always displays a certain string.
	 */
	void add_constant_text_column(Glib::ustring text, Glib::ustring header);

	/**
	 * Removes all the columns of the list. Use this when you want to
	 * start over and setup new columns.
	 */
	void remove_all_columns();

	/**
	 * Have the InstanceList widget display all the instances of a
	 * particular class.
	 */
	void enum_instances(OpenWBEM::String className);

	/**
	 * Have the InstanceList display CIM instances that are associated
	 * with a particular instance. Specify what association class is
	 * used in the associations.
	 */
	void associators(const OpenWBEM::CIMObjectPath &refObject,
			const OpenWBEM::String &assocClass);
	
	/**
	 * Gets the CIMObjectPath for the currently selected instance.
	 * This is useful if you have a menu option that must act on whatever
	 * item is currently selected in the result pane.
	 */
	OpenWBEM::CIMObjectPath get_selected_object();

	SigC::Signal1<void, OpenWBEM::CIMObjectPath> signal_activated;
	bool use_default_instance_activated_handler;

private:
	void populate_list(const std::vector<CIMInstanceHandle> &result);
	void new_list();

public:
	/**
	 * The column record used by the InstanceList widget.
	 */
	class ModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	public:
		ModelColumns()
		{
			add(classname);
			add(inst_handle);
		}
		Gtk::TreeModelColumn<Glib::ustring> classname;
		Gtk::TreeModelColumn<CIMInstanceHandle> inst_handle;
	};
	
	ModelColumns Columns;

private:
	class Column
	{
	public:
		virtual ~Column();

	protected:
		// PROPERTIES
		OpenWBEM::String PropertyName;
	
		// CONSTRUCTOR
		Column(OpenWBEM::String &propertyName)
			: PropertyName(propertyName) { }

	public:
		// METHODS
		virtual void setRowData(Gtk::TreeRow &row, CIMInstanceHandle inst)
			= 0;
	};
	
	class TextColumn : public Column
	{
	public:
		// PROPERTIES
		Gtk::TreeModelColumn<Glib::ustring> ModelColumn;
	
		// CONSTRUCTOR
		TextColumn(OpenWBEM::String &name,
				Gtk::TreeModelColumn<Glib::ustring> &model)
			: Column(name),
			  ModelColumn(model)  {	}
		
		// METHODS
		virtual void setRowData(Gtk::TreeRow &row, CIMInstanceHandle inst);
	};
	
	class ConstantColumn : public Column
	{
	public:
		// PROPERTIES
		Gtk::TreeModelColumn<Glib::ustring> ModelColumn;
		Glib::ustring ConstantText;
	
		// CONSTRUCTOR
		ConstantColumn(OpenWBEM::String &name,
					Gtk::TreeModelColumn<Glib::ustring> &model,
					Glib::ustring &text)
			: Column(name), ModelColumn(model), ConstantText(text) { }

		// METHODS
		virtual void setRowData(Gtk::TreeRow &row, CIMInstanceHandle inst);
	};
	
	std::vector<OpenWBEM::Reference<Column> > column_mappings;
	
protected:
	// HANDLERS
	virtual void on_row_activated(const Gtk::TreeModel::Path &path,
			Gtk::TreeViewColumn *column);
	virtual void on_instance_activated(CIMInstanceHandle inst);

	// OTHER METHODS
};

#endif /* INSTANCELIST_H */
