// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <gtkmm/window.h>
#include <gtkmm/box.h>
#include <gtkmm/menubar.h>
#include <gtkmm/menu.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/paned.h>
#include <gtkmm/treestore.h>
#include <openwbem/OW_CIMOMHandleIFC.hpp>
#include <openwbem/OW_CIMObjectPath.hpp>
#include "InstanceList.h"
#include "ScopeItem.h"

/**
 * The main window of the CIMBrowser application.
 */
class MainWindow : public Gtk::Window
{
private:
	class ScopeTreeView : public Gtk::TreeView
	{
	public:
		MainWindow *main_window;
		virtual bool on_button_press_event(GdkEventButton *event);
	};

	// WIDGETS
	Gtk::VBox vbox;
	Gtk::MenuBar menu_bar;
	Gtk::Menu menu_console, menu_scope, menu_result, menu_schema, menu_help;
	InstanceList result_list;
	Gtk::ScrolledWindow result_scrolled;
	Gtk::HPaned paned;
	ScopeTreeView scope_list;
	Gtk::ScrolledWindow scope_scrolled;
	Gtk::TreeModel::iterator scope_current;
	Glib::RefPtr<Gtk::TreeStore> scope_store;

public:
	/**
	 * Create a new main window.
	 */
	MainWindow();

	/**
	 * Fills in the scope pane with initial data. For now, this initial
	 * data is a hard-coded scope item that lists instances of
	 * CIM_Service. Eventually, this should fill in data by reading
	 * in a file.
	 */
	void init_scope_list();

	/**
	 * Enumerate the instance's of a class in the result pane.
	 */
	void enum_instances(const OpenWBEM::String &className);

	/**
	 * Accesses the main window's result list widget.
	 */
	InstanceList *get_result_list()
		{ return &result_list; }
	
	/**
	 * Adds a scope item under an existing item to the scope pane.
	 * The caller must identify what name is used. This name is compared
	 * to existing scope items, and the new item is only added if the
	 * name is not already used.
	 */
	void add_scope_item(Gtk::TreeRow &row, Glib::ustring name,
			ScopeItemRef scopeitem);

protected: // callbacks
	void on_menu_console_add_cim_object();
	void on_menu_console_add_container();
	void on_menu_console_quit();
	void on_menu_schema_find_class();
	void on_menu_help_about();

	/**
	 * A scope item that displays instance data in the result pane.
	 * The result data is shown in a table format, with customizable
	 * columns.
	 */
	class SimpleScopeItem : public ScopeItem
	{
	private:
		/**
		 * Describes what columns to display in the instance list.
		 */
		OpenWBEM::String columns;

	protected:
		SimpleScopeItem(const OpenWBEM::String &columns);
		void setup_columns(MainWindow *window);
	};
	/**
	 * A scope item that displays all instances of a particular CIM
	 * class.
	 */
	class ClassScopeItem : public SimpleScopeItem,
	  	public SigC::Object
	{
	private:
		/**
		 * The name of the class to enumerate in the result pane.
		 */
		OpenWBEM::String className;
		MainWindow *main_window;

	protected:
		virtual void init_actions_menu(Gtk::Menu &action_menu);
		void on_menu_new();

	public:
		bool expander;
		OpenWBEM::String expander_assocClass;
		ClassScopeItem(const OpenWBEM::String &className,
				const OpenWBEM::String &columns, MainWindow *window);
		virtual void activate();
		virtual void expand_row(Gtk::TreeModel::Row &row);
	};

	/**
	 * The column record class used by the scope tree store.
	 */
	class ScopeModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	public:
		/**
		 * Create a new column record.
		 */
		ScopeModelColumns()
		{
			add(name);
			add(scopeitem);
		}
		/**
		 * Scope item's name. This is the text that actually gets
		 * displayed in the scope pane.
		 */
		Gtk::TreeModelColumn<Glib::ustring> name;
		/**
		 * Pointer to the ScopeItem that handles this scope item.
		 */
		Gtk::TreeModelColumn<ScopeItemRef> scopeitem;
	};

	/**
	 * The column record used by the scope tree store.
	 */
	ScopeModelColumns ScopeColumns;

	virtual void on_scope_row_activated(const Gtk::TreeModel::Path &path,
		Gtk::TreeViewColumn *column);
};

#endif /* MAINWINDOW_H */
